'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var t = require('tap');
var ServerlessEndpoint = require('../test/serverless-endpoint');
var A = require('./serverless');

t.test('Adapter: Serverless', function (t) {
  t.plan(13);

  var arnie = A({ providers: [function (context) {
      context.anotherProvider = 'test';
      return context;
    }] });

  var actionNum = {
    taskA: 0,
    taskB: 0
  };
  var testExecutionOrder = function testExecutionOrder(group, order) {
    t.equal(actionNum[group] += 1, order, 'should run each action on request in order (' + group + ' ' + order + ')');
  };

  var taskA = [function (ctx) {
    t.assert(ctx.output, 'should provide output in context');
    t.assert(ctx.request, 'should provide request in context');
    t.assert(ctx.response, 'should provide reponse in context');
    t.equal(ctx.anotherProvider, 'test', 'should pass additional providers to function-tree');
    testExecutionOrder('taskA', 1);
    throw new Error('Error in taskA');
  }];

  var taskB = [function (_ref) {
    var props = _ref.props,
        path = _ref.path;

    testExecutionOrder('taskB', 1);
    return new _promise2.default(function (resolve) {
      setTimeout(function () {
        return resolve(path.goHere({ someOutput: true }));
      }, 50);
    });
  }, {
    dontGoHere: [function () {
      return t.fail('should NOT run this action');
    }],
    goHere: [function (_ref2) {
      var response = _ref2.response;

      testExecutionOrder('taskB', 2);
      response.body = 'text';
    }]
  }, function (ctx) {
    return testExecutionOrder('taskB', 3);
  }];

  var serverless = ServerlessEndpoint({
    '/error': arnie(taskA),
    '/success': arnie(taskB)
  });

  serverless.get('/error').end(function (error, result) {
    if (!error) {
      t.equal(result.statusCode, 500, 'should send status code 500 on unhandled error');
    }
  });

  serverless.get('/success').end(function (error, result) {
    if (!error) {
      t.equal(actionNum.taskB, 3, 'should complete taskB before request ends');
      t.equal(result.text, 'text', 'should send the body');
      t.equal(result.statusCode, 200, 'should send the status code');
      t.equal(result.headers['content-type'], 'text/plain; charset=utf-8', 'should send the content type');
    }
  });
});