'use strict';

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var get = require('get-value');
var set = require('set-value');

var Tag = function () {
  function Tag(tag, strings, values) {
    (0, _classCallCheck3.default)(this, Tag);

    this.type = tag;
    this.strings = strings;
    this.values = values;
  }
  /*
    Gets the path of the tag, where nested tags are evaluated
  */


  (0, _createClass3.default)(Tag, [{
    key: 'getPath',
    value: function getPath(context) {
      if (!context) {
        throw new Error('You can not grab the path from a Tag without context');
      }

      return this.populatePath(context);
    }
    /*
      Uses the path of the tag to get value in related context
    */

  }, {
    key: 'getValue',
    value: function getValue(context) {
      if (!context) {
        throw new Error('You can not grab a value from a Tag without context');
      }

      if (!context[this.type]) {
        throw new Error('You can not grab a value from a Tag without its provider');
      }

      var source = context[this.type];
      var path = this.getPath(context);
      return get(source, path);
    }
    /*
      Uses the path of the tag to set value in related context
    */

  }, {
    key: 'setValue',
    value: function setValue(context, value) {
      if (!context) {
        throw new Error('You can not set a value through a Tag without context');
      }
      if (!context[this.type]) {
        throw new Error('You can not set a value through a Tag without its provider');
      }
      var target = context[this.type];
      var path = this.getPath(context);
      return set(target, path, value);
    }
    /*
      Populates nested tags in the tags path
    */

  }, {
    key: 'populatePath',
    value: function populatePath(context) {
      var _this = this;

      return this.strings.reduce(function (currentPath, string, idx) {
        var valueTemplate = _this.values[idx];

        if (valueTemplate instanceof Tag) {
          return currentPath + string + valueTemplate.getValue(context);
        }

        return currentPath + string + (valueTemplate || '');
      }, '');
    }
    /*
      Produces a string representation of the tag
    */

  }, {
    key: 'toString',
    value: function toString() {
      return this.type + '`' + this.pathToString() + '`';
    }
    /*
      Produces a string representation of the path
    */

  }, {
    key: 'pathToString',
    value: function pathToString() {
      var _this2 = this;

      return this.strings.reduce(function (currentPath, string, idx) {
        var valueTemplate = _this2.values[idx];

        if (valueTemplate instanceof Tag) {
          return currentPath + string + '${' + valueTemplate.toString() + '}';
        }

        return currentPath + string + (valueTemplate || '');
      }, '');
    }
  }]);
  return Tag;
}();

module.exports = Tag;