'use strict';

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var supertest = require('supertest');
var url = require('url');
var qs = require('qs');
var pathToRegexp = require('path-to-regexp');

function ServerlessEndpoint(routes) {
  var router = createRouter(routes);
  var endpoint = createTestEndpoint(router);

  return endpoint;
}

function createTestEndpoint(router) {
  var endpoint = supertest.agent(function (request, res) {
    var body = [];
    request.on('data', function (chunk) {
      body.push(chunk);
    }).on('end', function () {
      body = Buffer.concat(body).toString();

      var awsEvent = createEvent(request, body);
      var awsContext = createContext();

      var route = router.get(awsEvent.path);
      awsEvent.pathParameters = route.params;

      route.handler(awsEvent, awsContext, function (error, response) {
        if (error) {
          res.statusCode = 500;
          res.end();
          return;
        }

        res.writeHead(response.statusCode, response.headers);
        res.end(response.body);
      });
    });
  });

  return endpoint;
}

function createRouter(routes) {
  var routeSpecs = (0, _keys2.default)(routes).map(function (route) {
    var keys = [];
    return {
      route: route,
      keys: keys,
      handler: routes[route],
      re: pathToRegexp(route, keys)
    };
  });

  return {
    get: function get(path) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        var _loop = function _loop() {
          var spec = _step.value;

          var result = spec.re.exec(path);
          if (result) {
            var handler = spec.handler;
            var params = result.slice(1).reduce(function (params, value, index) {
              var param = spec.keys[index];
              params[param.name] = value;
              return params;
            }, {});

            return {
              v: {
                params: params,
                handler: handler
              }
            };
          }
        };

        for (var _iterator = (0, _getIterator3.default)(routeSpecs), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var _ret = _loop();

          if ((typeof _ret === 'undefined' ? 'undefined' : (0, _typeof3.default)(_ret)) === "object") return _ret.v;
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      throw new Error('No route for path ' + path + ' defined!');
    }
  };
}

function createEvent(req, body) {
  var urlSpecs = url.parse(req.url);

  return {
    httpMethod: req.method,
    path: urlSpecs.pathname,
    headers: req.headers,
    queryStringParameters: qs.parse(urlSpecs.query),
    body: body
  };
}

function createContext() {
  return {};
}

module.exports = ServerlessEndpoint;